/***************************************************************************
Bascic file handling functions.

include <file.h>

Opens up Wav and CDR files, fills tloopinfo and tfileinfo structures and
returns success or not.

load_file(char *filename, tsong *song)

tloopinfo and tfileinfo need to allocated first.

Also includes functions to byte swap.

read_raw(FILE *fp, void *buf, int len)
read_cdr(FILE *fp, void *buf, int len)
write_raw(FILE *fp, void *buf, int len)
***************************************************************************/
#include <stdio.h>
#include <fcntl.h>
#include <limits.h>
#include <string.h>
#include "defs.h"
#include "dsp.h"

#define CDAUDIOSECTORSIZE 2352
#define	PCM_WAVE_FORMAT   	1
#define WW_BADOUTPUTFILE	1
#define WW_BADWRITEHEADER	2
#define	TRUE			1
#define	FALSE			0
#define BUFFERSIZE 1024

#define WR_BADALLOC		3
#define WR_BADSEEK		4
#define WR_BADRIFF		5
#define WR_BADWAVE		6
#define WR_BADFORMAT		7
#define WR_BADFORMATSIZE	8

#define WR_NOTPCMFORMAT		9
#define WR_NODATACHUNK		10
#define WR_BADFORMATDATA	11

typedef signed short        SINT16;                 /* 16 bits signed */

#define BYTES2SWAPPED(val) (\
		  (((val) >> 8) & 0x00ff) \
      |	 ((val) << 8) \
	)
#define SWAP2BYTES(arg) (arg) = BYTES2SWAPPED ((arg))
#define AS_S16(bufptr) ((SINT16 *) bufptr)

/**************************************************************************
* Functions to read and write from different formats 
* To swap or not to swap.
**************************************************************************/

long read_cdr(FILE *fp,void *buf,int len)
{
	long count;
	register int i;
	count = fread(buf, 1,len,fp);
		for (i = 0; i < (count / 2); i++)
			SWAP2BYTES (AS_S16(buf)[i]);
	return (count);
}

/**************************************************************************
* These two handle either wave or raw. 
**************************************************************************/
long read_raw(FILE *fp,void *buf,int len)
{
	return(fread(buf, 1,len,fp));
}

long write_raw(FILE *fp, void *buf,int len)
{
	return(fwrite(buf, 1,len, fp));
}

/***************************************************************************
*
* Load MessyWindows WAVE file
*
***************************************************************************/
typedef  struct
{	char    	RiffID [4] ;
	long    	RiffSize ;
	char    	WaveID [4] ;
	char    	FmtID  [4] ;
	long    	FmtSize ;
	short   	wFormatTag ;
	short   	chans ;
	long		srate ;
	long		seconds ;
	short		blkalign ;
	short		depth ;
	char		DataID [4] ;
	long		length ;
} wav_header ;

typedef  struct
{	long     dwSize ;
	short    wFormatTag ;
	short    wChannels ;
	long     dwSamplesPerSec ;
	long     dwAvgBytesPerSec ;
	short    wBlockAlign ;
	short    wBitsPerSample ;
} WAVEFORMAT ;


char* findchunk  (char* pstart, char* fourcc, size_t n)
{	char	*pend ;
	int		k, test ;

	pend = pstart + n ;

	while (pstart < pend)
	{ 	if (*pstart == *fourcc)       /* found match for first char*/
		{	test = TRUE ;
			for (k = 1 ; fourcc [k] != 0 ; k++)
				test = (test ? ( pstart [k] == fourcc [k] ) : FALSE) ;
			if (test)
				return  pstart ;
		} ; /* if*/
		pstart ++ ;
	} ; /* while lpstart*/

	return  NULL ;
}  /* findchuck*/

long  WaveReadHeader  (FILE *fp, tfileinfo *info)
{	
	static  WAVEFORMAT  waveformat;
	static	char   buffer [ BUFFERSIZE ] ;		/* Function is not reentrant.*/
	char*   ptr ;
	unsigned long  databytes ;
	int start = 0;

	rewind(fp);
	fread(&buffer, 1, BUFFERSIZE,fp) ;

	if (findchunk (buffer, "RIFF", BUFFERSIZE) != buffer) 
		return  WR_BADRIFF ;

	if (! findchunk (buffer, "WAVE", BUFFERSIZE)) 
		return  WR_BADWAVE ;

	ptr = findchunk (buffer, "fmt ", BUFFERSIZE) ;

	if (! ptr) 
		return  WR_BADFORMAT ;

	ptr += 4 ;	/* Move past "fmt ".*/
	memcpy (&waveformat, ptr, sizeof (WAVEFORMAT)) ;

	if (waveformat.dwSize < (sizeof (WAVEFORMAT) - sizeof(unsigned long)))
		return  WR_BADFORMATSIZE ;

	if (waveformat.wFormatTag != PCM_WAVE_FORMAT) 
		return  WR_NOTPCMFORMAT ;

	ptr = findchunk (buffer, "data", BUFFERSIZE) ;

	if (! ptr) 
		return  WR_NODATACHUNK ;

	ptr += 4 ;	/* Move past "data".*/
	memcpy (&databytes, ptr, sizeof (unsigned long)) ;

	start  = ((int) (ptr + 4)) - ((int) (&(buffer[0]))) ;
	if (waveformat.dwSamplesPerSec != waveformat.dwAvgBytesPerSec / 
												waveformat.wBlockAlign) 
		return  WR_BADFORMATDATA ;

	if (waveformat.dwSamplesPerSec != waveformat.dwAvgBytesPerSec / 
									waveformat.wChannels / 
							((waveformat.wBitsPerSample == 16) ? 2 : 1)) 
		return  WR_BADFORMATDATA ;
	/* Everything is now cool, so fill in output data.*/

	info->chans   = waveformat.wChannels ;
	info->srate= waveformat.dwSamplesPerSec ;
	info->depth = waveformat.wBitsPerSample ;
	info->blkalign = waveformat.wBlockAlign ;
	info->datalen	= databytes;

	return start;
}  /* WaveReadHeader*/

void write_header(FILE *fp,tfileinfo *info)
{
	wav_header header; 
	sprintf(header.RiffID,"RIFF");
	sprintf(header.WaveID,"WAVE");
	sprintf(header.FmtID,"fmt ");
	sprintf(header.DataID,"data");
	header.chans   = info->chans ;
	header.srate= info->srate ;
	header.depth = info->depth ;
	header.blkalign = info->blkalign ;
	header.RiffSize   = sizeof(wav_header);
	header.wFormatTag = PCM_WAVE_FORMAT ;
	header.seconds = info->srate*info->blkalign ;
	header.FmtSize = sizeof (WAVEFORMAT) - sizeof(long);
	header.length	= info->datalen;
	fwrite(&header,1,sizeof(wav_header),fp);
}
int load_wav(char *filename, tfileinfo *info, tloopinfo *loop)
{
	if((loop->fp = fopen(filename,"rb")) != NULL) {
		
		if ((info->datastart = WaveReadHeader( loop->fp, info )) > 20){
			loop->endpos = info->datalen/info->blkalign ; 
			loop->curpos = loop->startpos = info->datastart/info->blkalign; 
			fseek (loop->fp, info->datastart, SEEK_SET);
			loop->read = read_raw;
			return 0;
		}
		else
			return -1;
	}
  	else
		return -1;
}

int load_raw(char *filename, tfileinfo *info, tloopinfo *loop)
{
	if((loop->fp = fopen(filename,"rb")) != NULL) {
		info->srate = 44100;
		info->chans = 2;
		info->depth = 16;
		info->blkalign = 4;
		info->datastart = 0;
		loop->read = read_raw;
		fseek(loop->fp,0,SEEK_END);
		info->datalen = ftell(loop->fp);
		loop->curpos = loop->startpos = 0; 
		rewind(loop->fp);
		return 0;
	}
  	else
		return -1;
		
}

int load_cdr(char *filename, tfileinfo *info, tloopinfo *loop)
{
	if(load_raw(filename, info, loop) != -1) {
		loop->read = read_cdr;
		return 0;
	}
  	else
		return -1;
}

int load_file(char *filename, tsong *asong)
{
  if(strstr(filename,".wav") != NULL)   /*.WAV*/
    return load_wav(filename,&asong->info,&asong->loop);
  else if(strstr(filename,".cdr") != NULL)   /*.cdr*/
	return load_cdr(filename,&asong->info,&asong->loop);
  else if(strstr(filename,".raw") != NULL)   /*.raw*/
	return load_raw(filename,&asong->info,&asong->loop);
  else
	return -1;
}
/*
void open_out(char *path, tsong *song)
{
	memcpy(&recinfo,&song->info,sizeof(tfileinfo));
	recinfo.datalen = 0;
	recfp = fopen(path,"wb");
	write_header(recfp, &recinfo);
	rec = 0;
}

void close_out(void)
{
	rec = -1;
	fseek(recfp,0,SEEK_SET);
	write_header(recfp, &recinfo);
	fclose(recfp);
}
*/
void do_output(char *path,tsong *song)
{
	void *buffer;
	FILE *outfile;
	tfileinfo outinfo;
	long len;
	long count;
	long byteswrote = 0;
	outfile = fopen(path,"wb");
	memcpy(&outinfo,&song->info,sizeof(tfileinfo));
	write_header(outfile, &outinfo);
	buffer = malloc(CDAUDIOSECTORSIZE);

	len = (song->loop.endpos - song->loop.startpos)*
			song->info.blkalign;

	fseek( song->loop.fp, song->loop.startpos
						*song->info.blkalign, SEEK_SET);

	do{
		count =  song->loop.read( song->loop.fp, buffer, 
													CDAUDIOSECTORSIZE);
		byteswrote += write_raw( outfile, buffer, count);
	}while((len - byteswrote) >= CDAUDIOSECTORSIZE );

	count =  song->loop.read( song->loop.fp, buffer,(len - byteswrote));
	byteswrote += write_raw( outfile, buffer, count);
	outinfo.datalen = byteswrote;

	fseek( outfile, 0, SEEK_SET);
	write_header(outfile, &outinfo);
}

