#include <stdlib.h>
#include <curses.h>
#include <pthread.h>
#include <sys/soundcard.h>
#ifdef WITH_ESD
#include <esd.h>
#endif
#include "defs.h"
#include "dsp.h"
#include "file.h"
#include "misc.h"

#define DSP_BUF_SIZE 88200 

#ifdef WITH_ESD
#define BUF_SIZE 	256 
#else
#define BUF_SIZE 	DSP_BUF_SIZE 
#endif

int dsp_fd;
int bsize = BUF_SIZE;
int *buffer;
uint inc = 10000;
int y = 0;
int x; 
uint loops = 10; 
int speed = 10; 
uint end = 880000;
uint offset = 0;
float vol = 0.500;
tsong *song_ptr[MAX_SONGS];

void clear_area(int y1,int x1,int y2,int x2)
{
  register int a;
  for(y1 = y1;y1 <= y2;y1++)
    for(a = x1;a <= x2;a++)
    {
      move(y1,a);
      addch(' ');
    }
}
void *load_ram(tsong *song)
{
	long len, count;
	
	len = ( song->loop.endpos - song->loop.startpos ) *
				song->info.blkalign;
	if (len > 2*1024*1024)
		song->info.datalen = len = 2*1024*1024;

	if ((song->loop.buf = malloc(len)) != NULL){
		printf("Loading %s to RAM. Size is %i MB\n", song->sname,len/1024/1024);	
		fseek(song->loop.fp,song->loop.startpos*song->info.blkalign,SEEK_SET);
		count =  song->loop.read(song->loop.fp,song->loop.buf,len);
		if (count != len){
			free(song->loop.buf);
			song->loop.buf = NULL;
		}
	}
	return song->loop.buf;
}

void do_output(tsong *song)
{
  	int count = bsize*speed ;
	//	buffer = &song->loop.buf[y];
//		volumechange(buffer,vol,count);
		if ((count + y) > song->info.datalen)
			count = song->info.datalen - y;
		count = write(dsp_fd, &song->loop.buf[y], count);
	//	ioctl(dsp_fd,SNDCTL_DSP_SYNC,0);
}

void *do_loop(void *arg){
	int argc = (int) arg;
	int z;

	while(1){
		while(y<end){
			for(z=0;z<loops;z++){
				for(x=1;x<argc;x++){
					do_output(song_ptr[x]);
				}
			}
			y = y+inc;
		}
		y = offset;
	}
}
void print_out(void){
	clear_area(0,0,0,COLS);
	move(0,0);
	printw("Vol=%.3f increment=%i end=%i offset=%i loops=%i speed=%i",vol, inc, end,offset,loops,speed);
}


int main(int argc, char **argv)
{
	pthread_t pid;
	int ch;
	y=1;
	x=1;
	for (y=1;y<argc;y++){
		song_ptr[x] = malloc(sizeof( tsong));
		if (load_file(argv[y],song_ptr[x]) != -1){
			load_ram(song_ptr[x]);
			x++;
		}
	}

	if ((dsp_fd = open_sock(&song_ptr[1]->info)) == -1){
		printf("Error couldn't open output socket!\n");
		return(1);
	}	

	sleep(5);
	#ifndef WITH_ESD
	ioctl(dsp_fd, SNDCTL_DSP_GETBLKSIZE, &bsize);
	#endif
	buffer = malloc(bsize);
	pthread_create(&pid,NULL,do_loop,(void *)argc);
	initscr();
	cbreak();
	noecho();
	print_out();
	for(y=1;y<x;y++){
		mvprintw(3+y,0,"Loaded: %s", argv[y]);
	}
	y=0;
	while((ch = getch() ) != 'q'){
		switch(ch){
			case '/':
				vol = vol + 0.01;
			break;
			case '*':
				vol = vol - 0.01;
			break;
			case '1':
				loops++;
			break;
			case '2':
				if (loops != 1)
					loops--;
			break;
			case '3':
				speed ++;
			break;
			case '4':
				if (speed != 1)
					speed --;
			break;
			case 'i':
				mvprintw(1,0,"In edit mode.");
				while((ch = getch() ) != 'q'){
				switch(ch){
					case '+':
						inc = inc + 10000;
					break;
					case '-':
						inc = inc - 10000;
					break;
				}
				print_out();
				mvprintw(1,0,"In edit mode.");
				}
				move(1,0);
				clear_area(1,0,1,COLS);
			break;
			case 'e':
				mvprintw(1,0,"In edit mode.");
				while((ch = getch() ) != 'q'){
				switch(ch){
					case '+':
						end = end + 10000;
					break;
					case '-':
						end = end - 10000;
					break;
				}
				print_out();
				mvprintw(1,0,"In edit mode.");
				}
				move(1,0);
				clear_area(1,0,1,COLS);
			break;
			case 'o':
				mvprintw(1,0,"In edit mode.");
				while((ch = getch() ) != 'q'){
				switch(ch){
					case '+':
						offset = offset + 10000;
					break;
					case '-':
						offset = offset - 10000;
					break;
				}
				print_out();
				mvprintw(1,0,"In edit mode.");
				}
				move(1,0);
				clear_area(1,0,1,COLS);
			break;
		}
		print_out();
	}
	pthread_cancel(pid);
	close(dsp_fd);
	for(x=1;x<argc;x++)
		close(song_ptr[x]->loop.fp);
	endwin();
	return 1;

}
